
import struct
import serial
# from crccheck import …


def _break(ser):
  baud = ser.baudrate
  ser.baudrate = baud/2
  ser.write(bytes([0]))
  ser.flush()
  ser.baudrate = baud


class CyanBus:

  header = b'fxCyan'

  @staticmethod
  def makePacket(payload: bytes) -> bytes:
    'Wraps the payload in the standardized `fxCyan`-envelope'

    length = struct.pack('>H', len(payload))  # format = big-endian uint16_t
    crc = CyanBus.crc(payload)

    return CyanBus.header + length + payload + crc

  @staticmethod
  def crc(data: bytes) -> bytes:
    'Calculates and returns the CRC sum of the data'

    # TODO: implement!
    return bytes.fromhex('13371337')

  @staticmethod
  def sendRawPacket(data: bytes, port: str = '/dev/ttyUSB0', baud: int = 115200, sendBreak: bool = False):
    with serial.Serial(port=port, baudrate=baud, bytesize=8, stopbits=serial.STOPBITS_ONE) as ser:
      ser.write(data)
      ser.flush()
      if sendBreak: _break(ser)
      print(f'Send bytes[ {len(data)} ] long packet')

  @staticmethod
  def sendPacket(payload: bytes, port: str = '/dev/ttyUSB0', baud: int = 115200, sendBreak: bool = False):
    data = CyanBus.makePacket(payload)
    CyanBus.sendRawPacket(data, port=port, baud=baud, sendBreak=sendBreak)



class DebugSender:

  def __init__(self, port: str = '/dev/ttyUSB0', baud: int = 115200):
    self.port = port
    self.baud = baud

  def sendData(self, data: bytes):
    with serial.Serial(port=self.port, baudrate=self.baud, bytesize=8, stopbits=serial.STOPBITS_ONE) as ser:
      ser.write(data)
      ser.flush()

  def sendBreak(self):
    with serial.Serial(port=self.port, baudrate=self.baud, bytesize=8, stopbits=serial.STOPBITS_ONE) as ser:
      ser.baudrate = self.baud/2
      ser.write(bytes([0]))
      ser.flush()
      ser.baudrate = self.baud

  def sendPacket(self, payload: bytes, sendBreak: bool = False):
    CyanBus.sendPacket(payload=payload, port=self.port, baud=self.baud, sendBreak=sendBreak)

  def sendGarbagePacket(self, payload: bytes, garbage: bytes = b'GaRbAgE'):
    self.sendData(garbage)
    self.sendPacket(payload)

  def loopGarbagePacket(self, payload: bytes = b'Hallo, Welt!', garbage: bytes = b'GaRbAgE'):
    from time import sleep

    while True:
      self.sendGarbagePacket(payload, garbage)
      sleep(1)

  def loopPacket(self, payload: bytes = b'Hallo, Welt!'):
    from time import sleep

    while True:
      for _ in range(10):
        self.sendPacket(payload, False)
        sleep(1)
      self.sendPacket(payload, True)
      sleep(1)


cb = DebugSender(port='/dev/ttyUSB5')

